local collision_mask_util = require("collision-mask-util")

local DoNotAlterTypesList = {
	["spider-leg"] = true,
	["car"] = true,
	["locomotive"] = true,
	["cargo-wagon"] = true,
	["fluid-wagon"] = true,
	["artillery-wagon"] = true,
	["tree"] = true,
	["unit"] = true,
	["cliff"] = true,
	["simple-entity"] = true,
	["pipe"] = true,
	["character"] = true,
	["wall"] = true,
	["gate"] = true,
	["inserter"] = true,
	["arithmetic-combinator"] = true,
	["decider-combinator"] = true,
	["constant-combinator"] = true,
	["tile"] = true,
}
local DoNotAlterNamesList = {
	["small-electric-pole"] = true,
	["lab"] = true,
	["assembling-machine-1"] = true,
	["assembling-machine-2"] = true,
	["assembling-machine-3"] = true,
	["roboport"] = true,
	["stone-furnace"] = true,
}

local function CheckNextUpgrade(checkme, first, make_exempt)
	if not checkme or checkme.name == first then return make_exempt end
	if CheckNextUpgrade(data.raw[checkme.type][checkme.next_upgrade], first or checkme.name, make_exempt or DoNotAlterNamesList[checkme.name]) then
		DoNotAlterNamesList[checkme.name] = true
		if checkme.NOTSqueakThrough_oldCollision_box then checkme.collision_box = checkme.NOTSqueakThrough_oldCollision_box end
		return true
	else
		return false
	end
end

local function AllowedToAlter(checkme)
	if DoNotAlterTypesList[checkme.type] then return false end
	CheckNextUpgrade(checkme)
	if DoNotAlterNamesList[checkme.name] then return false end
	return true
end

local function AdjustAwayFromZero(number)
	if number % 0.5 ~= 0 then
		local coeff = 0.1
		if number > 0 then
			local c = math.ceil(number)
			if (c - number) < 0.5 then
				c = c - coeff
			else
				c = c - 0.5 - coeff
			end
			if c > number then number = c end
		elseif number < 0 then
			local f = math.floor(number)
			if (f - number) > -0.5 then
				f = f + coeff
			else
				f = f + 0.5 + coeff
			end
			if f < number then number = f end
		end
	end
	return number
end

local prototypes = collision_mask_util.collect_prototypes_colliding_with_mask(collision_mask_util.get_mask(data.raw.character.character))
for _, prototype in pairs(prototypes) do
	-- log("Canidate: ['" .. prototype.type .. "']['" .. prototype.name .. "']")
	if prototype.collision_box and not prototype.ignore_NOTSqueakThrough and AllowedToAlter(prototype) then
		-- log("Accepted")
		local new_collision_box = {{},{}}
		new_collision_box[1][1] = AdjustAwayFromZero(prototype.collision_box[1][1])
		new_collision_box[1][2] = AdjustAwayFromZero(prototype.collision_box[1][2])
		new_collision_box[2][1] = AdjustAwayFromZero(prototype.collision_box[2][1])
		new_collision_box[2][2] = AdjustAwayFromZero(prototype.collision_box[2][2])

		-- Prevents marking entities as altered by this mod even if it doesn't actually change anything
		if new_collision_box[1][1] ~= prototype.collision_box[1][1] or new_collision_box[1][2] ~= prototype.collision_box[1][2] or
			new_collision_box[2][1] ~= prototype.collision_box[2][1] or new_collision_box[2][2] ~= prototype.collision_box[2][2] then
			prototype.NOTSqueakThrough_oldCollision_box = prototype.collision_box -- save it incase we need to revert
			prototype.collision_box = new_collision_box
			-- log("Altered")
		end
	end
end